using UnityEngine;

/// <summary>
/// Allows setting the timescale to values outside the usual allowable range.
/// By Kuler
///
/// Thanks to AcidBubbles for locating the TimeControl field.
/// </summary>
public class TimescalePause : MVRScript
{
    private TimeControl _timeControl;

    // The timescale before the plugin started up
    private float _initialTimeScale = 1;
    // How much longer the slowmo/realtime button's effects will hold
    private float _buttonEffectDuration = 0;
    // Time to reset back to after the slowmo/realtime button expires
    private float _returnToTime = 1;

    private JSONStorableFloat _timeScaleJSON;

    public override void Init()
    {
        _initialTimeScale = Time.timeScale;

        _timeControl = GameObject.FindObjectOfType<TimeControl>();
        
        _timeScaleJSON = new JSONStorableFloat(
          "Time Scale",
          1f,
          val => {
            SetTimescale(val);
          }, 0.000f, 10f
        ) { isStorable = false };

        RegisterFloat(_timeScaleJSON);
        CreateSlider(_timeScaleJSON).label = "Time Scale (Set Only)";
        _timeScaleJSON.valNoCallback = _timeControl.currentScale;

        CreateButton("0.0x").button.onClick.AddListener(() => {
            // Za Warudo!
            SetTimescale(0.0f);
        });
        CreateButton("0.001x").button.onClick.AddListener(() => {
            SetTimescale(0.001f);
        });
        CreateButton("0.01x").button.onClick.AddListener(() => {
            SetTimescale(0.01f);
        });
        CreateButton("0.1x").button.onClick.AddListener(() => {
            SetTimescale(0.1f);
        });
        CreateButton("1.0x").button.onClick.AddListener(() => {
            SetTimescale(1.0f);
        });
        CreateButton("2.0x").button.onClick.AddListener(() => {
            SetTimescale(2.0f);
        });
        CreateButton("10.0x").button.onClick.AddListener(() => {
            SetTimescale(10.0f);
        });
        CreateButton("Realtime for 0.1 seconds").button.onClick.AddListener(() => {
            SetCustomTimeForDuration(1.0f, 0.1f);
        });
        CreateButton("Realtime for 1 second").button.onClick.AddListener(() => {
            SetCustomTimeForDuration(1.0f, 1.0f);
        });
        CreateButton("Realtime for 5 seconds").button.onClick.AddListener(() => {
            SetCustomTimeForDuration(1.0f, 5.0f);
        });

        SetupInfoText(this,
            "<size=40><b>"+
            "<color=#324e7a>Timescale P a u s e</color>" +
            "</b></size>\n" +
            "<b>To toggle between realtime and paused, press 0.</b>\n\n" +
            "Allows temporarily changing the Timescale (normally located under the 'Click for more options' Menu) to freeze time, go faster than realtime speed, or anywhere inbetween.\n\n" +
            "The fastest timescale is 10x normal speed.\n" +
            "<color=#FF0000>Based on your physics rate, physics update cap, and machine specs, you may not actually reach this speed.</color>\n\n" +
            "Use the Realtime buttons to advance time at 1x speed for a short duration.",
            750.0f,
            rightSide: true
        );
    }

    void OnDestroy() {
        SetTimescale(_initialTimeScale);
    }

    protected void Update() {
        // Time.deltaTime
        // Time.fixedDeltaTime
        if (_buttonEffectDuration > 0)
        {
            _buttonEffectDuration = Mathf.Max(0, _buttonEffectDuration - Time.fixedDeltaTime);
            if (_buttonEffectDuration == 0)
            {
                SetTimescale(_returnToTime);
            }
        }

        if (Input.GetKeyDown(KeyCode.Alpha0))
        {
            SetTimescale(Time.timeScale == 0 ? 1 : 0);
        }
    }

    private void SetCustomTimeForDuration(float customTime, float duration) {
        // Debounce
        if (_buttonEffectDuration > 0)
        {
            return;
        }

        _returnToTime = _timeScaleJSON.val;
        _buttonEffectDuration = duration;
        SetTimescale(customTime);
    }

    private void SetTimescale(float timeScale) {
        if (_timeScaleJSON.val != timeScale)
        {
            _timeScaleJSON.val = timeScale;
        }
        _timeControl.currentScale = timeScale;
        Time.timeScale = timeScale;
    }

    private static JSONStorableString SetupInfoText(MVRScript script, string text, float height, bool rightSide)
    {
        JSONStorableString storable = new JSONStorableString("Info", text);
        UIDynamic textfield = script.CreateTextField(storable, rightSide);
        textfield.height = height;
        return storable;
    }
}
